from utilities import align_parser, assembly, snv
from utilities.align_parsers import exonerate,blat
from optparse import OptionParser
import sys, re

class Bubble:

    ID = None
    branches = None
    alleles = None
    mapped2contig = None
    mapped2genome = None
    same_contig = None
    same_genome = None
    
    def __init__(self, ID):
        self.ID = ID
        self.branches = []
        self.alleles = {}
        self.mapped2contig = True
        self.mapped2genome = True
        self.same_contig = True
        self.same_genome = True

    def screen(self):
        for branch in self.branches:
            if not branch.contig:
                self.mapped2contig = False

            if not branch.genome_align:
                self.mapped2genome = False
            

        if self.mapped2contig:
            contig = self.branches[0].contig
            contig_pos = self.branches[0].contig_pos

            for i in range(1,len(self.branches)):
                if not self.branches[i].contig == contig:
                    self.same_contig = False
                if not self.branches[i].contig_pos[0] == contig_pos[0] or not self.branches[i].contig_pos[1] == contig_pos[1]:
                    self.same_contig = False

        else:
            self.same_contig = False

        if self.mapped2genome:
            target = self.branches[0].genome_align.target
            tstart = self.branches[0].genome_align.tstart
            tend = self.branches[0].genome_align.tend

            for i in range(1,len(self.branches)):
                if not self.branches[i].genome_align.target == target:
                    self.same_genome = False
                if not self.branches[i].genome_align.tstart == tstart or not self.branches[i].genome_align.tend == tend:
                    self.same_genome = False

        else:
            self.same_genome = False

class Branch:

    name = None
    length = None
    coverage = None
    bubble = None
    genome_align = None
    contig = None
    contig_pos = None

    def __init__(self, name, length=None, coverage=None):
        self.name = name
        self.length = length
        self.coverage = coverage

    def details(self):
        contig = "NA"
        genome = "NA"
        
        if self.contig and self.contig_pos:
            contig = "%s %s-%s" % (self.contig, self.contig_pos[0], self.contig_pos[1])

        if self.genome_align:
            genome = "%s %s-%s" % (self.genome_align.target, self.genome_align.tstart, self.genome_align.tend)

        return "%s contig:%s genome:%s" % (self.name, contig, genome)

class BubbleAnalyzer:

    fa_file = None
    k = None
    bubbles = None
    branches_dict = None
    branch2bubbles = None
    log_file = None
    aligner = None

    def __init__(self, fa_file, k=None, log_file=None):
        self.fa_file = fa_file
        self.k = k
        self.bubbles = []
        self.branches_dict = {}
        self.log_file = log_file

    def create_bubbles(self, contigs):
        "convert bubble contigs file to bubble objects"
    
        for contig in contigs:
            m = re.search('(\d+)(\w)', contig.num)
            if m:
                branch = Branch(contig.num, contig.length, contig.coverage)
                self.branches_dict[branch.name] = branch
            
                if m.group(2) == 'A':
                    bubble = Bubble(m.group(1))
                    branch.bubble = bubble
                    self.bubbles.append(bubble)

                bubble.branches.append(branch)
                
    def aligns2bubbles(self, genome_aligns, bubbles):
        "move genomic alignments to bubbles"

        branch_dict = {}
        for bubble in self.bubbles:
            for branch in bubble.branches:
                branch_dict[branch.name] = branch

        for a in genome_aligns:
            branch = a.query.split(' ')[0]
            self.branches_dict[branch].genome_align = a


    def bubbles2contigs(self, contig_aligns):
        for a in contig_aligns:
            branch = a.query.split(' ')[0]
            contig = a.target.split(' ')[0]

            print branch, contig, a.tstart, a.tend
            self.branches_dict[branch].contig = contig
            self.branches_dict[branch].contig_pos = [a.tstart, a.tend]



    def screen(self, genome_aligns_file, contig_aligns_file, aligner=None):
        if aligner == "exonerate":
            genome_aligns = align_parser.parse(genome_aligns_file, "exonerate", {'bestn':1, 'unique':True, 'match':90})
            self.aligns2bubbles(genome_aligns, self.bubbles)

            contig_aligns = align_parser.parse(contig_aligns_file, "exonerate", {'count':1})
            self.bubbles2contigs(contig_aligns)

            for bubble in self.bubbles:
                bubble.screen()

        elif aligner == "blat":
            genome_aligns = align_parser.parse(genome_aligns_file, "blat", {'bestn':1, 'unique':True, 'match':90})

            self.aligns2bubbles(genome_aligns, self.bubbles)

            contig_aligns = align_parser.parse(contig_aligns_file, "blat", {'count':1})
            self.bubbles2contigs(contig_aligns)

            for bubble in self.bubbles:
                bubble.screen()
            
    def find_snvs(self, genome_aligns_file, contig_aligns_file, output=None, aligner=None, blatblast=None):
        asm = assembly.Assembly(self.k, fasta=self.fa_file)
        contigs = asm.get_contigs()
        self.create_bubbles(contigs)
        self.screen(genome_aligns_file, contig_aligns_file, aligner=aligner)

        all_snvs = []
        blat_aligns = []
        for bubble in self.bubbles:
            if bubble.same_contig and bubble.same_genome:
                for branch in bubble.branches:
                    if aligner == "exonerate":
                        #snvs = branch.get_snvs()
                        snvs = exonerate.get_snvs(branch.genome_align)

                        for snv in snvs:
                            snv.var_len = branch.genome_align.query_len
                            snv.from_end = min(int(snv.var_start) - int(branch.genome_align.qstart), int(branch.genome_align.qend) - int(snv.var_end))
                        
                        all_snvs.extend(snvs)
                    elif aligner == "blat":
                        blat_aligns.append(branch.genome_align)

        if aligner == "blat":
            hsps = blat.get_hsps(blatblast, blat_aligns)
            hsps_used = {}

            count_block = 0
            count_hsp = 0
            for align in blat_aligns:
                snvs, hsps_found = blat.get_snvs(align, hsps, hsps_used)
                count_hsp += hsps_found
                count_block += len(align.blocks)

                for snv in snvs:
                    snv.var_len = align.query_len
                    snv.from_end = min(int(snv.var_start) - int(align.qstart), int(align.qend) - int(snv.var_end))

                all_snvs.extend(snvs)

            sys.stderr.write("blat snvs blocks:%d hsps:%d\n" % (count_block, count_hsp))

        if output:
            self.output_snvs(output, all_snvs)

    def output_snvs(self, output, snvs):
        out = open(output, 'w')
        out.write("\t".join(['type', 'chr', 'chr_start', 'chr_end', 'strand', 'ctg', 'ctg_len', 'ctg_start', 'ctg_end', 'len', 'change', 'from_end']) + "\n")

        for snv in snvs:
            out.write(snv.tab())

        out.close()

    def log(self):
        out = open(self.log_file, 'w')

        out.write("total bubbles: %d\n" % (len(self.bubbles)))
        out.write("good bubbles: %d\n" % (len([b for b in self.bubbles if b.same_contig and b.same_genome])))
        out.write("bad bubbles: %d\n" % (len([b for b in self.bubbles if not b.same_contig or not b.same_genome])))

        for bubble in self.bubbles:
            if bubble.same_contig and bubble.same_genome:
                for branch in bubble.branches:
                    out.write("good " + branch.details() + "\n")
        
        for bubble in self.bubbles:
            if not bubble.same_contig or not bubble.same_genome:
                for branch in bubble.branches:
                    out.write("bad " + branch.details() + "\n")

        out.close()
        

if __name__ == '__main__':
    usage = "Usage: %prog k bubbles_file genome_align_file contig_align_file aligner"
    parser = OptionParser(usage=usage)
    parser.add_option("-n", "--bestn", dest="bestn", help="best hit filter", type='int')
    parser.add_option("-i", "--identity", dest="identity", help="identity filter", type='float')
    parser.add_option("-m", "--match", dest="match_percent", help="match length percentage", type='float')
    parser.add_option("-u", "--unique", dest="unique", help="unique", action="store_true", default=False)
    parser.add_option("-o", "--out", dest="out", help="output /snv file, with proper file extension")
    parser.add_option("-l", "--log", dest="log", help="log file")
    parser.add_option("-b", "--blast", dest="blast", help="blat blast output for snv detection")

    (options, args) = parser.parse_args()

    filters = {}
    if len(args) == 5:
        if options.bestn:
            filters['bestn'] = options.bestn
        if options.identity:
            filters['identity'] = options.identity
        if options.match_percent:
            filters['match'] = options.match_percent
        if options.unique:
            filters['unique'] = True
    
        ba = BubbleAnalyzer(args[1], k=args[0], log_file=options.log)
        ba.find_snvs(args[2], args[3], output=options.out, aligner=args[4].lower(), blatblast=options.blast)

        if options.log:
            ba.log()
    else:
        parser.error("incorrect number of arguments")

