import pprint
import os
from optparse import OptionParser

class Assembly:
    #abyss assembly data"

    k = None
    sets = []
    pets = []
    contigs = []
    loc = None
    fasta = None
    set_file = None
    pet_file = None
    
    #k,version,
    def __init__(self, k, loc=None, set_file=None, pet_file=None, fasta=None, sequence=False):
        self.k = k
        self.loc = loc
        self.set_file = set_file
        self.pet_file = pet_file
        self.fasta = fasta
        
        if loc:
            (self.set_file, self.pet_file) = self.get_fasta(loc)
            
    def get_fasta(self):
        "given directory(loc), find SET and PET files"

        (self.set_file, self.pet_file) = (None, None)
        if self.loc:
            #files = filter(os.path.isdir, os.listdir(self.loc))
            files = os.listdir(self.loc)
            #spltnames = map(os.path.splitext, files)
            #print self.loc
            #print files
            #dash4 = [name for name,ext in spltnames if name.endswith('-4') and ext == '.fa']
            dash4 = [f for f in files if f.endswith('-4.fa') and not f.startswith('snp') and not f.startswith('contig') and not f.startswith('bubbles')]
            dashcontigs = [f for f in files if f.endswith('-contigs.fa')]
            print dash4
            print dashcontigs

            #check if only 1 file is extracted for "xx-4.fa" and "yy-contigs.fa" and "xx"="yy" (same prefix)
            if len(dash4) == len(dashcontigs) == 1 and dash4[0].split('-')[0] ==dashcontigs[0].split('-')[0] :
                (self.set_file, self.pet_file) = ['/'.join((self.loc, f)) for f in (dash4[0], dashcontigs[0])]

        return self

    def get_contigs(self, which=None, ids=[], length=0, len_range=[], sequence=False):
        "get contig data"
        
        contigs = []

        if which:
            if which == 'set':
                fa = self.set_file
            elif which == 'pet':
                fa = self.pet_file
        else:
            if self.pet_file:
                fa = self.pet_file
            elif self.set_file:
                fa = self.set_file
            else:
                fa = self.fasta

        if fa and os.path.exists(fa):
            ff = open(fa, 'r')
            seq = ""
            last_contig = None
            for line in ff.readlines():
                #header
                if line[0] == '>':
                    if last_contig:
                        if len(seq) > 0:
                            last_contig.sequence = seq
                        contigs.append(last_contig)
                    seq = ""
                    
                    cols = line.rstrip('\n').lstrip('>').split(' ')

                    good = True
                    if ids and cols[0] not in ids:
                        good = False

                    if good:
                        if length and int(length) > 0 and int(cols[1]) != int(length):
                            good = False

                        if len_range:
                            #proper range given
                            if int(len_range[1]) > int(len_range[0]):
                                if (int(cols[1]) < int(len_range[0])) or (int(cols[1]) > int(len_range[1])):
                                    good = False
                            #first number less than second number, indicates check minimum
                            elif int(cols[1]) < int(len_range[0]):
                                good = False
                    if good:
                        last_contig = Contig(cols[0], length=int(cols[1]), coverage=int(cols[2]))
                    else:
                        last_contig = None
                    
                elif sequence and line[0] != ' ':
                    #print seq
                    seq += line.rstrip('\n')

            if last_contig:
                if len(seq) > 0:
                    last_contig.sequence = seq
                contigs.append(last_contig)
            seq = ""
            
            if which == 'set':
                [ctg for ctg in contigs]
            elif which == 'pet':
                [ctg for ctg in contigs]

        return contigs
    

class Contig:
    num = 0
    length = 0
    coverage = 0
    set = False
    pet = False
    sequence = ""
    children = []
    align = []
    k = None

    def __init__(self, num, set=False, pet=False, length=0, coverage=0, k=None):
        self.num = num
        self.set = set
        self.pet = pet
        self.length = length
        self.coverage = coverage
        self.k = k

    def fasta(self):
        line = ">" + str(self.num) + " " + str(self.length) + " " + str(self.coverage) + "\n"
        line += self.sequence + "\n"

        return line

    def normalized_coverage(self):
        if int(self.coverage) >= 0 and self.length and self.k:
            return "%.1f" % (float(self.coverage) / (float(self.length) - float(self.k) + 1.0))
        else:
            return None

if __name__ == '__main__':
    usage = "Usage: %prog k-val [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("-d", "--dir", dest="dir", help="directory of assembly")
    parser.add_option("-s", "--set", dest="set", help="get set contigs", action="store_true", default=False)
    parser.add_option("-p", "--pet", dest="pet", help="get pet contigs", action="store_true", default=False)
    parser.add_option("-f", "--fasta", dest="fasta", help="fasta file")
    parser.add_option("-r", "--range", dest="range", help="size range, e.g. 100 0 (all contigs >= 100bp)", nargs=2)
    parser.add_option("-o", "--out", dest="out", help="output fasta")
    parser.add_option("-l", "--len", dest="len", help="contig length")
    parser.add_option("-C", "--count", dest="count", help="count contigs", action="store_true", default=False)
    parser.add_option("-c", "--contigs", dest="contigs", help="get contigs, e.g. 1/2/3")
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("no k-value given")
    elif not options.fasta and not options.dir:
        parser.error("no input fasta file given")
    elif options.dir and (not options.set and not options.pet):
        parser.error("set/pet not indicated")
    else:
        assembly = Assembly(k=args[0])

        if options.fasta:
            assembly.fasta = options.fasta
        elif options.dir:
            assembly.loc = options.dir

        if options.count:
            contigs = assembly.get_contigs(length=options.len, len_range=options.range)
            print len(contigs)
            
        elif options.out:
            if options.fasta:
                if options.contigs:
                    ids = options.contigs.split('/')
                    contigs =  assembly.get_contigs(length=options.len, len_range=options.range, sequence=True, ids=ids)
                else:
                    contigs = assembly.get_contigs(length=options.len, len_range=options.range, sequence=True)
            elif options.dir:
                if options.set:
                    which = 'set'
                else:
                    which = 'pet'
                assembly.loc = options.dir
                contigs = assembly.get_fasta().get_contigs(length=options.len, which=which, len_range=options.range, sequence=True)
                
            print len(contigs)

            out = open(options.out, 'w')
            for ctg in contigs:
                out.write(ctg.fasta())
            out.close()
                
